package com.getbubblenow.android.fragment;

import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.airbnb.lottie.LottieAnimationView;
import com.getbubblenow.android.R;
import com.getbubblenow.android.activity.BaseActivityBubble;
import com.getbubblenow.android.repository.DataRepository;
import com.getbubblenow.android.viewmodel.ProgressLoadingViewModel;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.ViewModelProvider;

public class ProgressLoadingDialogFragment extends DialogFragment implements DataRepository.NetworkProgressStatusInterface {

    private ProgressLoadingViewModel progressLoadingViewModel;
    private TextView progressTitle;
    private LottieAnimationView lottieAnimationView;

    public static ProgressLoadingDialogFragment newInstance() {
        ProgressLoadingDialogFragment fragment = new ProgressLoadingDialogFragment();
        Bundle args = new Bundle();
        fragment.setArguments(args);
        return fragment;
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        final Dialog dialog = new Dialog(getActivity(), android.R.style.Theme_DeviceDefault_Light_NoActionBar_Fullscreen) {
            @Override
            public void onBackPressed() {
                if (getActivity().getSupportFragmentManager().findFragmentByTag(BaseActivityBubble.PROGRESS_TAG) != null) {
                    getActivity().onBackPressed();
                }
                super.onBackPressed();
            }
        };
        dialog.setCanceledOnTouchOutside(false);
        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        return dialog;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.progress_loading_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        progressLoadingViewModel = new ViewModelProvider(this).get(ProgressLoadingViewModel.class);
        lottieAnimationView = view.findViewById(R.id.network_progress);
        progressTitle = view.findViewById(R.id.progress_title);
        progressLoadingViewModel.setNetworkProgressStateInterface(this);
    }

    @Override public void onCreate(@Nullable final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override public void setProgress(long percent) {
        final long textProgress = percent;
        if(percent%10>=5){
            percent = percent + 10 - (percent%10);
        }
        else {
            percent -= (percent % 10);
        }
        lottieAnimationView.setProgress(percent/100f);
        progressTitle.setText(textProgress+"%");
    }
}
